"""Registry registrar REST API views."""
import asyncio
from typing import Any, Awaitable, Coroutine, Dict, List

from asgiref.sync import sync_to_async
from django.utils.translation import gettext as _
from ninja import Router
from regal.exceptions import RegistrarDoesNotExist

from ferda.api import Error, check_permissions
from ferda.backend import LOGGER, REGISTRAR
from ferda.constants import LOGGER_SESSION_KEY, LogEntryType, LoggerResult
from ferda.settings import SETTINGS

from .schema import Registrar

router = Router(tags=['registry', 'registrar'])


async def _await_with_semaphore(call: Awaitable, semaphore: asyncio.Semaphore) -> Coroutine:
    async with semaphore:
        return await call


async def _get_registrar(handle: str, *, with_credit: bool, semaphore: asyncio.Semaphore) -> Dict[str, Any]:
    """Get a registrar info, zone_access and credit in case user has permission for it."""
    calls = [REGISTRAR.get_registrar_info(handle), REGISTRAR.get_registrar_zone_access(handle)]
    if with_credit:
        calls.append(REGISTRAR.get_registrar_credit(handle))

    results = await asyncio.gather(*[_await_with_semaphore(call, semaphore) for call in calls])
    return {'info': results[0], 'zone_access': results[1], 'credit': results[2] if with_credit else None}


@router.get('/', url_name='list_registrars', response=List[Registrar])
async def list_registrars(request):
    """Get a list of all registrars with their info, zone access and credit."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])

    log_entry = LOGGER.create(
        LogEntryType.REGISTRAR_LIST,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
    )

    with log_entry:
        handles = await REGISTRAR.get_registrar_handles()
        semaphore = asyncio.Semaphore(SETTINGS.concurrent_grpc_calls_limit)
        with_credit = await sync_to_async(request.user.has_perm)('ferda.can_view_registrar_credit')
        registrar_list = await asyncio.gather(*[
            _get_registrar(handle, with_credit=with_credit, semaphore=semaphore) for handle in handles
        ])

        log_entry.result = LoggerResult.SUCCESS
        log_entry.references['registrar'] = handles

    return registrar_list


@router.get('/{handle}/', url_name='get_registrar', response={200: Registrar, 404: Error})
async def get_registrar(request, handle: str):
    """Get a single registrar info, zone access and credit."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])

    log_entry = LOGGER.create(
        LogEntryType.REGISTRAR_DETAIL,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        references={'registrar': handle},
    )

    with log_entry:
        try:
            semaphore = asyncio.Semaphore(SETTINGS.concurrent_grpc_calls_limit)
            with_credit = await sync_to_async(request.user.has_perm)('ferda.can_view_registrar_credit')
            registrar = await _get_registrar(handle, with_credit=with_credit, semaphore=semaphore)

        except RegistrarDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail=_('Registrar does not exist'))

        log_entry.result = LoggerResult.SUCCESS
        return registrar
