from decimal import Decimal
from unittest.mock import patch

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.test import TestCase, override_settings, tag
from django.urls import reverse
from frgal.utils import make_awaitable
from regal import Registrar
from regal.exceptions import RegistrarDoesNotExist

from ferda.constants import LogEntryType, LoggerResult
from ferda.tests.utils import LoggerMixin

User = get_user_model()


@tag('api', 'registry')
@patch('ferda.registry.registrar.api.REGISTRAR')
@override_settings(ROOT_URLCONF='ferda.tests.urls')
class ListRegistrarsTest(LoggerMixin, TestCase):
    def setUp(self):
        super().setUp()
        user = User.objects.create_superuser('admin')  # type: ignore
        self.client.force_login(user)

    def test_get_registrars_list(self, client):
        client.get_registrar_handles.return_value = make_awaitable(['REG-TEST_A'])
        client.get_registrar_info.return_value = make_awaitable(
            Registrar(registrar_handle='REG-TEST_A', name='Test name')
        )
        client.get_registrar_zone_access.return_value = make_awaitable({'cz': True})
        client.get_registrar_credit.return_value = make_awaitable({'cz': Decimal('1000.00')})

        response = self.client.get(reverse('api-1:list_registrars'))

        self.assertLog(LogEntryType.REGISTRAR_LIST, LoggerResult.SUCCESS, references={'registrar': ['REG-TEST_A']})
        self.assertEqual(response.status_code, 200)
        self.maxDiff = None
        self.assertEqual(response.json(), [{
            'info': {
                'registrar_handle': 'REG-TEST_A',
                'name': 'Test name',
                'organization': '',
                'place': None,
                'telephone': None,
                'fax': None,
                'emails': [],
                'url': None,
                'company_registration_number': None,
                'vat_identification_number': None,
                'is_system_registrar': False,
                'variable_symbol': '',
                'payment_memo_regex': '',
                'is_vat_payer': False,
            },
            'credit': {'cz': '1000.00'},
            'zone_access': {'cz': True},
        }])

    def test_get_registrar_list_credit_authorized_user(self, client):
        client.get_registrar_handles.return_value = make_awaitable(['REG-TEST_A'])
        client.get_registrar_info.return_value = make_awaitable(
            Registrar(registrar_handle='REG-TEST_A', name='Test name')
        )
        client.get_registrar_zone_access.return_value = make_awaitable({'cz': True})
        client.get_registrar_credit.return_value = make_awaitable({'cz': Decimal('1000.00')})

        response = self.client.get(reverse('api-1:list_registrars'))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()[0]['credit'], {'cz': '1000.00'})

    def test_get_registrar_list_credit_unauthorized_user(self, client):
        unauthorized_user = User.objects.create_user('johndoe', 'john@example.com', 'password')
        unauthorized_user.user_permissions.set(
            Permission.objects.filter(codename__in=('can_view_registry',)))
        self.client.force_login(unauthorized_user)

        client.get_registrar_handles.return_value = make_awaitable(['REG-TEST_A'])
        client.get_registrar_info.return_value = make_awaitable(
            Registrar(registrar_handle='REG-TEST_A', name='Test name')
        )
        client.get_registrar_zone_access.return_value = make_awaitable({'cz': True})

        response = self.client.get(reverse('api-1:list_registrars'))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()[0]['credit'], None)


@tag('api', 'registry')
@patch('ferda.registry.registrar.api.REGISTRAR')
@override_settings(ROOT_URLCONF='ferda.tests.urls')
class GetRegistrarTest(LoggerMixin, TestCase):
    def setUp(self):
        super().setUp()
        user = User.objects.create_superuser('admin')  # type: ignore
        self.client.force_login(user)

    def test_get_registrar(self, client):
        client.get_registrar_info.return_value = make_awaitable(
            Registrar(registrar_handle='REG-TEST_A', name='Test name')
        )
        client.get_registrar_credit.return_value = make_awaitable({'cz': Decimal('1000.00')})
        client.get_registrar_zone_access.return_value = make_awaitable({'cz': True})

        response = self.client.get(reverse('api-1:get_registrar', kwargs={'handle': 'REG-TEST_A'}))

        self.assertLog(
            LogEntryType.REGISTRAR_DETAIL,
            LoggerResult.SUCCESS,
            input_references={'registrar': 'REG-TEST_A'}
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), {
            'info': {
                'name': 'Test name',
                'registrar_handle': 'REG-TEST_A',
                'company_registration_number': None,
                'emails': [],
                'fax': None,
                'is_system_registrar': False,
                'is_vat_payer': False,
                'organization': '',
                'payment_memo_regex': '',
                'place': None,
                'telephone': None,
                'url': None,
                'variable_symbol': '',
                'vat_identification_number': None,
            },
            'credit': {'cz': '1000.00'},
            'zone_access': {'cz': True},
        })

    def test_get_registrar_does_not_exist(self, client):
        client.get_registrar_info.side_effect = RegistrarDoesNotExist

        response = self.client.get(reverse('api-1:get_registrar', kwargs={'handle': 'REG-TEST_A'}))

        self.assertLog(
            LogEntryType.REGISTRAR_DETAIL,
            LoggerResult.FAIL,
            input_references={'registrar': 'REG-TEST_A'}
        )
        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.json(), {'detail': 'Registrar does not exist'})

    def test_get_registrar_credit_authorized_user(self, client):
        client.get_registrar_info.return_value = make_awaitable(
            Registrar(registrar_handle='REG-TEST_A', name='Test name')
        )
        client.get_registrar_credit.return_value = make_awaitable({'cz': Decimal('1000.00')})
        client.get_registrar_zone_access.return_value = make_awaitable({'cz': True})

        response = self.client.get(reverse('api-1:get_registrar', kwargs={'handle': 'REG-TEST_A'}))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()['credit'], {'cz': '1000.00'})

    def test_get_registrar_credit_unauthorized_user(self, client):
        unauthorized_user = User.objects.create_user('johndoe', 'john@example.com', 'password')
        unauthorized_user.user_permissions.set(
            Permission.objects.filter(codename__in=('can_view_registry',)))
        self.client.force_login(unauthorized_user)

        client.get_registrar_info.return_value = make_awaitable(
            Registrar(registrar_handle='REG-TEST_A', name='Test name')
        )
        client.get_registrar_zone_access.return_value = make_awaitable({'cz': True})

        response = self.client.get(reverse('api-1:get_registrar', kwargs={'handle': 'REG-TEST_A'}))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()['credit'], None)
